package com.ruoyi.web.controller.api;

import com.alibaba.fastjson.JSONObject;
import com.alibaba.fastjson.serializer.SerializerFeature;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.ruoyi.common.annotation.Log;
import com.ruoyi.common.config.RuoYiConfig;
import com.ruoyi.common.constant.Constants;
import com.ruoyi.common.core.controller.BaseController;
import com.ruoyi.common.core.domain.AjaxResult;
import com.ruoyi.common.core.page.TableDataInfo;
import com.ruoyi.common.enums.BusinessType;
import com.ruoyi.common.exception.ServiceException;
import com.ruoyi.common.utils.DateUtils;
import com.ruoyi.common.utils.RequestParamsUtil;
import com.ruoyi.common.utils.SecurityUtils;
import com.ruoyi.common.utils.StringUtils;
import com.ruoyi.common.utils.file.FileUtils;
import com.ruoyi.common.utils.http.HttpUtils;
import com.ruoyi.system.domain.*;
import com.ruoyi.system.domain.req.*;
import com.ruoyi.system.domain.resp.PatientHistoryRecordResp;
import com.ruoyi.system.domain.resp.TRecordResp;
import com.ruoyi.system.service.*;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.apache.commons.collections4.CollectionUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;
import sun.misc.BASE64Encoder;

import javax.annotation.Resource;
import javax.imageio.stream.FileImageInputStream;
import java.io.ByteArrayOutputStream;
import java.io.File;
import java.util.*;
import java.util.stream.Collectors;

/**
 * @author zhangcl
 * @date 2022/8/5 17:42
 */
@Api("app接口")
@RestController
@RequestMapping("/api/app")
public class AppController extends BaseController
{
    @Autowired
    private ITRecordService tRecordService;

    @Autowired
    private ITHospitalService tHospitalService;

    @Resource
    private ITDoctorService tDoctorService;

    @Resource
    private ITPatientService tPatientService;

    @Resource
    private ITPatientHospitalDoctorService itPatientHospitalDoctorService;

    @Resource
    private ISysConfigService configService;

    @Resource
    private ITRecordHistoryService recordHistoryService;

    @Resource
    private ITImageService imageService;

    @Value("${aiPost.ip}")
    private String ip;

    @Value("${aiPost.port}")
    private String port;

    @Value("${aiPost.url}")
    private String url;

    /**
     * 完成AI诊断
     */
    @ApiOperation("完成AI诊断")
    @Log(title = "App诊断记录状态", businessType = BusinessType.UPDATE)
    @GetMapping("/ai/finish/{id}")
    public AjaxResult finish(@PathVariable("id") Long id)
    {
        TRecord record = tRecordService.selectTRecordById(id);
        if (record != null)
        {
            record.setStatus("1");
        }
        return toAjax(tRecordService.updateTRecord(record));

    }

    /**
     * 医院列表
     */
    @ApiOperation("医院列表")
    @GetMapping("/hospital/list")
    public AjaxResult hospitalList()
    {
        List<THospital> tHospitals = tHospitalService.selectTHospitalList(new THospital());
        return AjaxResult.success(tHospitals);
    }


    /**
     * 医生信息获取
     *
     * @param doctorId 医生id
     * @return {@code AjaxResult}
     */
    @ApiOperation("医生信息获取")
    @GetMapping("/doctor/info")
    @Log(title = "获取医生信息", businessType = BusinessType.OTHER)
    public AjaxResult doctorInformation(Long doctorId)
    {
        return AjaxResult.success(tDoctorService.selectAppDoctorDetailById(doctorId));
    }

    /**
     * 患者预约
     *
     * @param appPatientHospitalDoctor 应用病人医院医生
     * @return {@code AjaxResult}
     */
    @ApiOperation("小程序患者预约")
    @Log(title = "小程序患者预约", businessType = BusinessType.INSERT)
    @GetMapping("/patient/makeAnAppointment")
    public AjaxResult patientsToMakeAnAppointment(AppPatientHospitalDoctor appPatientHospitalDoctor)
    {
        // 查询该患者当天是否有预约记录
        TRecord tRecord = new TRecord();
        tRecord.setPatientId(getUserId());
        tRecord.setHospitalId(appPatientHospitalDoctor.getHospitalId());
        tRecord.setStatus("0");
        tRecord.setDoctorId(appPatientHospitalDoctor.getDoctorId());
        tRecord.setAppointmentTime(DateUtils.dateTime(new Date()));
        List<TRecord> tRecords = tRecordService.selectTRecordList(tRecord);
        if (!CollectionUtils.isEmpty(tRecords)) {
            AjaxResult ajaxResult = AjaxResult.error();
            ajaxResult.put("msg", "您今天已登记过信息！");
            ajaxResult.put("isBook", true);
            return ajaxResult;
        }

        // 患者预约，新增t_record中的一条记录;判断关联表是否存在当前关系
        TPatientHospitalDoctor tPatientHospitalDoctor = new TPatientHospitalDoctor();
        BeanUtils.copyProperties(appPatientHospitalDoctor, tPatientHospitalDoctor);
        tPatientHospitalDoctor.setPatientId(getUserId());
        tRecord.setCreateBy(getUsername());
        tRecordService.insertTRecord(tRecord);
        List<TPatientHospitalDoctor> tPHDs = itPatientHospitalDoctorService.selectTPatientHospitalDoctorList(tPatientHospitalDoctor);
        if (tPHDs.isEmpty())
        {
            itPatientHospitalDoctorService.insertTPatientHospitalDoctor(tPatientHospitalDoctor);
        }
        TRecord queryRecord = new TRecord();
        queryRecord.setId(tRecord.getId());
        List<PatientHistoryRecordResp> resp = tRecordService.queryPatientHistoryRecord(queryRecord);
        AjaxResult ajaxResult = AjaxResult.success();
        ajaxResult.put("record", CollectionUtils.isEmpty(resp) ? null : resp.get(0));
        ajaxResult.put("msg", "登记成功");
        return ajaxResult;
    }

    @PutMapping("/record/ai/finish/{id}")
    @ApiOperation("患者记录预约到已诊状态转换")
    @Log(title = "患者记录预约到已诊状态转换", businessType = BusinessType.UPDATE)
    public AjaxResult updateStatus(@PathVariable("id") Long id)
    {
        if (id == null)
        {
            return AjaxResult.error("未获取到记录id");
        }
        TRecord tRecord = new TRecord();
        tRecord.setId(id);
        tRecord.setStatus("1");
        tRecordService.updateTRecord(tRecord);
        return AjaxResult.success("更新状态成功", null);
    }

    @GetMapping(value = "/config/ai/refreshtime")
    //@Log(title = "获取AI结果刷新时间接口")
    @ApiOperation("获取AI结果刷新时间接口")
    public AjaxResult getConfigKey()
    {
        String configKey = "t.ai.refreshtime";
        return AjaxResult.success("获取成功" ,configService.selectConfigByKey(configKey));
    }

    /**
     * 诊断记录上传
     *
     * @return {@code AjaxResult}
     */
    @ApiOperation("诊断记录上传")
    @Log(title = "App诊断记录上传", businessType = BusinessType.UPDATE)
    @PostMapping("/record/diagnostic/upload")
    public AjaxResult diagnosticRecordsUploaded(@RequestBody AppRecordReq appRecordReq)
    {
        //覆盖recode记录前，先备份数据
        //备份历史记录
        recordHistoryService.saveHistory(appRecordReq.getId());

        TRecord tRecord = new TRecord();
        BeanUtils.copyProperties(appRecordReq, tRecord);
        if (null == tRecord.getAiResultModifyTime()) {
            tRecord.setAiResultModifyTime(DateUtils.getNowDate());
        }
        return AjaxResult.success(tRecordService.updateTRecord(tRecord));
    }

    /**
     * 添加病人
     *
     * @param appPatientReq App端添加患者入参实体
     * @return {@code AjaxResult}
     */
    @ApiOperation("添加患者")
    @Log(title = "App患者添加", businessType = BusinessType.INSERT)
    @PostMapping("/patient/add")
    public AjaxResult addPatient(@RequestBody @Validated AppPatientReq appPatientReq)
    {
        logger.info("add patient param is {} ", appPatientReq);
        TPatient tPatient = new TPatient();
        BeanUtils.copyProperties(appPatientReq, tPatient);
        tPatient.setDelFlag("0");
        // 查询患者信息
        TPatient currentPatient = tPatientService.queryTPatientByPhoneAndName(tPatient);
        if (Objects.nonNull(currentPatient))
        {
            //患者信息已存在 新增诊断记录
            TRecord tRecord = new TRecord();
            tRecord.setHospitalId(appPatientReq.getHospitalId());
            tRecord.setPatientId(currentPatient.getId());
            tRecord.setDoctorId(appPatientReq.getDoctorId());
            tRecord.setStatus("0");
            tRecord.setCreateBy(getUsername());
            tRecordService.insertTRecord(tRecord);
            return AjaxResult.success("手机号已注册");
        }
        tPatientService.insertTPatient(tPatient);
        Long identifier = 10000000L + tPatient.getId();
        tPatient.setIdentifier(String.valueOf(identifier));
        tPatientService.updateTPatient(tPatient);
        TPatientHospitalDoctor tPatientHospitalDoctor = new TPatientHospitalDoctor();
        tPatientHospitalDoctor.setHospitalId(appPatientReq.getHospitalId());
        tPatientHospitalDoctor.setDoctorId(appPatientReq.getDoctorId());
        tPatientHospitalDoctor.setPatientId(tPatient.getId());
        List<TPatientHospitalDoctor> tPatientHospitalDoctors = itPatientHospitalDoctorService.selectTPatientHospitalDoctorList(tPatientHospitalDoctor);
        if (tPatientHospitalDoctors.isEmpty())
        {
            itPatientHospitalDoctorService.insertTPatientHospitalDoctor(tPatientHospitalDoctor);
        }


        TRecord tRecord = new TRecord();
        tRecord.setHospitalId(appPatientReq.getHospitalId());
        tRecord.setPatientId(tPatient.getId());
        tRecord.setDoctorId(appPatientReq.getDoctorId());
        tRecord.setStatus("0");
        tRecord.setCreateBy(getUsername());
        tRecordService.insertTRecord(tRecord);
        List<TPatientHospitalDoctor> tPHDs = itPatientHospitalDoctorService.selectTPatientHospitalDoctorList(tPatientHospitalDoctor);
        if (tPHDs.isEmpty())
        {
            itPatientHospitalDoctorService.insertTPatientHospitalDoctor(tPatientHospitalDoctor);
        }

        return AjaxResult.success();
    }

    /**
     * 修改患者
     *
     * @param appPatientReq App端添加患者入参实体
     * @return {@code AjaxResult}
     */
    @ApiOperation("患者信息修改")
    @Log(title = "App患者修改", businessType = BusinessType.UPDATE)
    @PostMapping("/patient/modify")
    public AjaxResult patientsWithModified(@RequestBody @Validated AppPatientReq appPatientReq)
    {
        logger.info("modify patient param is {} ", appPatientReq);
        TPatient tPatient = new TPatient();
        BeanUtils.copyProperties(appPatientReq, tPatient);
        tPatient.setDelFlag("0");
        TPatient currentPatient = tPatientService.queryTPatientByPhoneAndName(tPatient);
        if (Objects.nonNull(currentPatient) && currentPatient.getId().compareTo(appPatientReq.getId()) != 0) {
            return AjaxResult.error("手机号下已存在该患者。");
        }
        return AjaxResult.success(tPatientService.updateTPatient(tPatient));
    }

    @ApiOperation("获取患者详细信息")
    @GetMapping("/patient/info/{id}")
    @Log(title = "获取患者详细信息", businessType = BusinessType.OTHER)
    public AjaxResult getPatientInfo2(@PathVariable("id") Long id)
    {

        return AjaxResult.success(tPatientService.selectTPatientById(id));
    }


    /**
     * 诊断记录列表
     *
     * @param patientId 应用病人医院医生
     * @return {@code AjaxResult}
     */
    @ApiOperation("患者诊断记录历史列表")
    @GetMapping("/record/list/{patientId}")
    @Log(title = "患者诊断记录历史列表", businessType = BusinessType.OTHER)
    public AjaxResult diagnosisOfRecordList(@PathVariable("patientId") Long patientId)
    {
        TPatientHospitalDoctor tPatientHospitalDoctor = new TPatientHospitalDoctor();
        tPatientHospitalDoctor.setPatientId(patientId);
        List<TRecordResp> tRecords = tRecordService.queryRecordListByPHDids(tPatientHospitalDoctor);
        List<TRecordResp> tRecordResps = tRecords.stream().filter(record -> "1".equals(record.getStatus())).collect(Collectors.toList());
        if (tRecordResps.isEmpty()) {
           return AjaxResult.success(new ArrayList<TRecordResp>());
        }
        List<TRecordResp> respList = tRecordResps.stream()
                .sorted(Comparator.comparing(TRecordResp::getUploadTime, Comparator.nullsLast(Date::compareTo)).reversed()).collect(Collectors.toList());
        return AjaxResult.success(respList);
    }

    /**
     * 患者诊断记录信息细节
     *
     * @param id id
     * @return {@code AjaxResult}
     */
    @ApiOperation("患者诊断记录信息详情")
    @GetMapping("/record/info/{id}")
    @Log(title = "患者诊断记录信息详情", businessType = BusinessType.OTHER)
    public AjaxResult patientsWithDiagnosisOfRecordInformationDetails(@PathVariable("id") Long id)
    {
        Map<String,Object> data = new HashMap<>();
        data.put("result",tRecordService.selectById(id));
        TImage tImage = new TImage();
        tImage.setThirdId(id);
        List<TImage> images = imageService.selectTImageList(tImage);
        data.put("image", images);
        return AjaxResult.success(data);
    }

    /**
     * 患者已诊列表
     *
     * @param recordResultReq 记录结果要求
     * @return {@code AjaxResult}
     */
    @ApiOperation("患者已诊列表")
    @GetMapping("/record/diagnosedList")
    @Log(title = "患者已诊列表", businessType = BusinessType.OTHER)
    public TableDataInfo patientsDiagnosedList(TRecordResultReq recordResultReq)
    {
//        recordResultReq.setStatus("1");
        startPage();
        return getDataTable(tRecordService.selectTRecordListWithoutResult(recordResultReq));
    }

    @ApiOperation("首页患者列表")
    @GetMapping("/home/patient/list")
    @Log(title = "查询首页患者列表", businessType = BusinessType.OTHER)
    public TableDataInfo queryHomePatientsList(TRecordResultReq recordResultReq)
    {
//        recordResultReq.setStatus("1");
        startPage();
        List<TRecordResp> tRecordResps = tRecordService.queryHomePatientList(recordResultReq);
        return getDataTable(tRecordResps);
    }

    /**
     * 病人等待名单
     *
     * @param recordResultReq 记录结果要求
     * @return {@code AjaxResult}
     */
    @ApiOperation("患者候诊列表")
    @GetMapping("/record/waitingList")
    @Log(title = "查询患者候诊列表", businessType = BusinessType.OTHER)
    public TableDataInfo patientsWaitingList(TRecordResultReq recordResultReq)
    {
//        recordResultReq.setStatus("0");
        startPage();
        return getDataTable(tRecordService.selectTRecordListWithoutResult(recordResultReq));
    }

    @ApiOperation("小程序获取患者详细信息")
    @GetMapping("/patient/info")
    @Log(title = "小程序获取患者详细信息", businessType = BusinessType.OTHER)
    public AjaxResult getPatientInfo()
    {
        return AjaxResult.success(tPatientService.selectTPatientById(getUserId()));
    }

    @GetMapping("/patient/record/historyData")
    @ApiOperation("查询患者历史数据图表")
    @Log(title = "查询患者历史数据图表", businessType = BusinessType.OTHER)
    public AjaxResult getHistoryRecordList(AppRecordReq appRecordReq)
    {
        TRecord tRecord = new TRecord();
        BeanUtils.copyProperties(appRecordReq, tRecord);
        return tRecordService.getHistoryList(tRecord);
    }

    @PostMapping("/ai/{recordId}")
    @ApiOperation("ai接口")
    @Log(title = "调用ai接口", businessType = BusinessType.OTHER)
    public AjaxResult aiPost(@PathVariable("recordId") Long recordId, @RequestBody AIPostReqEx aiPostReqEx)
    {
        AiPostReq aiPostReq = new AiPostReq();
        BeanUtils.copyProperties(aiPostReqEx, aiPostReq);
        logger.info("update recordId is '{}' status. system time is '{}'", recordId, System.currentTimeMillis());
        TRecord tRecord = new TRecord();
        tRecord.setId(recordId);
        // APP跳转页面，需要立刻修改记录状态
        tRecord.setStatus("2");
        tRecord.setAppDealTime(aiPostReqEx.getAppDealTime());
        logger.info("update record status is '{}', record is '{}'",tRecord.getStatus(), tRecord);
        tRecordService.updateTRecord(tRecord);
        //删除同一次预约调用多次ai接口的图片数据
        logger.info("aiPost param is {}", aiPostReq);
        imageService.deleteTImageByThirdId(recordId);
        saveImage(aiPostReq,recordId);
        ObjectMapper mapper = new ObjectMapper();
        String URL = RequestParamsUtil.HTTP + ip + RequestParamsUtil.SEPARATOR + port + url;
        JSONObject jsonObject = new JSONObject();
        String result;
        Date start;
        Date end;
        Map<String,Object> data = new HashMap<>();
        try {
            start = new Date();
            if (aiPostReq.getUsr() != null) {
                aiPostReq.getUsr().setTs_submit_ai(start.getTime());
            }
            logger.info("ai param is {}", aiPostReq);
            result = HttpUtils.sendPost(URL, mapper.writeValueAsString(delNullData(aiPostReq)));
            end = new Date();
        } catch (Exception e) {
            // AI接口异常，需要重置当前记录状态
            updateRecordStatus(recordId, "0");
            logger.error("AI result exception, msg is {}", e.getMessage());
            return AjaxResult.error("AI result exception, please try again!", data);
        }
        if (StringUtils.isEmpty(result)) {
            // AI接口非正常返回，需要重置当前记录状态
            updateRecordStatus(recordId, "0");
            return AjaxResult.error(200,"AI result is null. please try again!");
        } else {
            JSONObject resultJB = JSONObject.parseObject(result);
            Integer code = resultJB.getInteger("code");
            if (code != 200 && code != 204) {
                // AI接口非正常返回，需要重置当前记录状态
                updateRecordStatus(recordId, "0");
                throw new ServiceException(resultJB.getString("msg"));
            }
            if (code == 204) {
                // AI接口非正常返回，需要重置当前记录状态
                updateRecordStatus(recordId, "3");
                return AjaxResult.error(code, resultJB.getString("msg"));
            }
            // 正常返回，记录状态置为“1”
            tRecord.setStatus("1");
            jsonObject = removeBase64Image(resultJB, recordId);
            tRecord.setUploadTime(start);
            tRecord.setAiResultReturnTime(end);
            tRecord.setAiResult(JSONObject.toJSONString(jsonObject, SerializerFeature.WriteMapNullValue));
            tRecord.setAiResult2(JSONObject.toJSONString(jsonObject, SerializerFeature.WriteMapNullValue));
            logger.info("update record status is '{}', record is '{}'",tRecord.getStatus(), tRecord);
            tRecordService.updateTRecord(tRecord);
        }
        data.put("result",jsonObject);
        TImage tImage = new TImage();
        tImage.setThirdId(recordId);
        List<TImage> images = imageService.selectTImageList(tImage);
        data.put("image", images);
        return AjaxResult.success(data);
    }

    private void updateRecordStatus(Long recordId, String status) {
        TRecord tRecord1 = new TRecord();
        tRecord1.setId(recordId);
        tRecord1.setStatus(status);
        tRecordService.updateTRecord(tRecord1);
    }

    @GetMapping("/image/{recordId}")
    @ApiOperation("获取图片接口")
    @Log(title = "获取图片接口", businessType = BusinessType.OTHER)
    public AjaxResult getAiImage(@PathVariable("recordId") Long recordId){
        if(recordId == null){
            return AjaxResult.error("未获取到参数");
        }
        TImage tImage = new TImage();
        tImage.setThirdId(recordId);
        List<TImage> images = imageService.selectTImageList(tImage);
        return AjaxResult.success(images);
    }


    @PostMapping("/ai2/{recordId}")
    @Log(title = "ai接口备用")
    @ApiOperation("ai接口备用")
    public AjaxResult aiPost2(@PathVariable("recordId") Long recordId, @RequestBody AiPostReq aiPostReq)
    {
        //删除同一次预约调用多次ai接口的图片数据
        imageService.deleteTImageByThirdId(recordId);
//        saveImage(aiPostReq,recordId);
//        revertImageToBase64(aiPostReq);
        ObjectMapper mapper = new ObjectMapper();
        String URL = RequestParamsUtil.HTTP + ip + RequestParamsUtil.SEPARATOR + port + url;
        JSONObject jsonObject = new JSONObject();
        TRecord tRecord = new TRecord();
        tRecord.setId(recordId);
        tRecord.setStatus("1");
        tRecordService.updateTRecord(tRecord);
        String result;
        Date start;
        Date end;
        try
        {
            start = new Date();
            result = HttpUtils.sendPost(URL, mapper.writeValueAsString(delNullData(aiPostReq)));
            end = new Date();
        }
        catch (Exception e)
        {
            throw new ServiceException(e.getMessage());
        }
        if (!StringUtils.isEmpty(result))
        {
            JSONObject data = JSONObject.parseObject(result);
            if(data.getInteger("code") != 200){
                TRecord tRecord1 = new TRecord();
                tRecord1.setId(recordId);
                tRecord1.setStatus("0");
                tRecordService.updateTRecord(tRecord1);
                throw new ServiceException(data.getString("msg"));
            }
            jsonObject = removeBase64Image(data,recordId);
            tRecord.setUploadTime(start);
            tRecord.setAiResultReturnTime(end);
            tRecord.setAiResult(JSONObject.toJSONString(jsonObject, SerializerFeature.WriteMapNullValue));
            tRecord.setAiResult2(JSONObject.toJSONString(jsonObject, SerializerFeature.WriteMapNullValue));
            tRecordService.updateTRecord(tRecord);
        }
        Map<String,Object> data = new HashMap<>();
        data.put("result",jsonObject);
        TImage tImage = new TImage();
        tImage.setThirdId(recordId);
        List<TImage> images = imageService.selectTImageList(tImage);
        data.put("image", images);
        return AjaxResult.success(data);
    }

    public  AiPostReq revertImageToBase64(AiPostReq aiPostReq,Long recordId)
    {
        Base64ResizedRmgSm base64_resized_img_sm = aiPostReq.getBase64_resized_img_sm();

        JSONObject jsonObject = JSONObject.parseObject(JSONObject.toJSONString(base64_resized_img_sm));


        jsonObject.forEach((key,value)->{
            if(value instanceof String){
                jsonObject.put(key,toBase64((String) value));
                TImage tImage = new TImage();
                tImage.setThirdId(recordId);
                tImage.setPath((String) value);
                tImage.setCreateTime(new Date());
                tImage.setName("sm_"+key);
                imageService.insertTImage(tImage);
            }
            if(value instanceof List){
                ((List<?>) value).forEach(o -> {
                    jsonObject.put(key,toBase64((String) o));
                    TImage tImage = new TImage();
                    tImage.setThirdId(recordId);
                    tImage.setPath((String) o);
                    tImage.setCreateTime(new Date());
                    tImage.setName("sm_"+ key+"_" +((List<?>) value).indexOf(o));
                    imageService.insertTImage(tImage);
                });

            }
            Base64ResizedRmgSm base64ResizedRmgSm = JSONObject.parseObject(jsonObject.toJSONString(), Base64ResizedRmgSm.class);
            aiPostReq.setBase64_resized_img_sm(base64ResizedRmgSm);
        });
        return null;
    }

    public String toBase64(String path){
        return  sourceToBase64(RuoYiConfig.getProfile() + StringUtils.substringAfter(path, Constants.RESOURCE_PREFIX));
    }


    public static String sourceToBase64(String path) {
        byte[] data = null;
        FileImageInputStream input = null;
        try {
            input = new FileImageInputStream(new File(path));
            ByteArrayOutputStream output = new ByteArrayOutputStream();
            byte[] buf = new byte[1024];
            int numBytesRead = 0;
            while((numBytesRead = input.read(buf)) != -1){
                output.write(buf, 0, numBytesRead);
            }
            data = output.toByteArray();
            output.close();
            input.close();
        } catch (Exception e) {
            e.printStackTrace();
        }
        BASE64Encoder base64Encoder = new BASE64Encoder();
        return base64Encoder.encode(data);
    }
    private void saveImage(AiPostReq aiPostReq,Long recordId)
    {

        Base64ResizedRmgSm base64_resized_img_sm = aiPostReq.getBase64_resized_img_sm();
        Base64ResizedRmgSm base64_resized_img_sx = aiPostReq.getBase64_resized_img_sx();
        Map<String, String> smMap = imageKey(base64_resized_img_sm,"sm");
        Map<String, String> sxMap = imageKey(base64_resized_img_sx, "sx");
        smMap.putAll(sxMap);
        smMap.forEach((name,value)->{
            if(!StringUtils.isEmpty(value)){
                TImage tImage = new TImage();
                tImage.setThirdId(recordId);
                tImage.setCreateTime(new Date());
                tImage.setName(name);
                String path=FileUtils.GenerateImage(value,name);
                tImage.setPath(path);
                imageService.insertTImage(tImage);
            }
        });
    }

    public Map<String,String> imageKey(Base64ResizedRmgSm param,String key){
        Map<String,String> map = new HashMap<>();
        if(!StringUtils.isEmpty(param.getFront())){
            map.put(key+"_front",param.getFront());
        }
        if(!StringUtils.isEmpty(param.getLeft())){
            map.put(key+"_left",param.getLeft());
        }
        if(!StringUtils.isEmpty(param.getRight())){
            map.put(key+"_right",param.getRight());
        }
        if(!CollectionUtils.isEmpty(param.getOther())){
            for (int i = 0; i < param.getOther().size(); i++)
            {
                map.put(key+"_other_"+i,param.getOther().get(i));
            }
        }
        return map;

    }

    public JSONObject removeBase64Image(JSONObject jsonObject,Long recordId)
    {
        List<String> jsonKeyList = new ArrayList<String>(){{
            add("st_avg_yellow");
            add("img_sz");
            add("img_sz_yudian");
            add("img_sz_yuban");
            add("img_sz_liewen");
            add("img_st");
            add("img_sx");
            add("img_sx_vein");
            add("sz_avg_red");
            add("sz_avg_purple");
            add("sx_vein_avg_purple");
        }};
        try {
            doRemoveBase64Image(jsonObject.getJSONObject("data"), "data", jsonKeyList, recordId);
        }
        catch (Exception e)
        {
            TRecord tRecord = new TRecord();
            tRecord.setId(recordId);
            tRecord.setStatus("0");
            tRecordService.updateTRecord(tRecord);
            throw new ServiceException("JSON解析错误:" + e.getMessage());
        }
        return jsonObject;
    }

    public JSONObject delNullData(AiPostReq aiPostReq){
        JSONObject jsonObject = JSONObject.parseObject(JSONObject.toJSONString(aiPostReq));
        return jsonObject;

    }

    /**
     * 移除base64
     *
     * @param obj       obj
     * @param listName  listName
     * @param imageKeys imageKeys
     * @param recordId  recordId
     */
    public void doRemoveBase64Image(Object obj, String listName, List<String> imageKeys, Long recordId) {
        // 遍历json
        if (obj instanceof JSONObject) {
            for (Map.Entry<String, Object> entry : ((JSONObject) obj).entrySet()) {
                // 如果key包含在图片
                if (imageKeys.contains(entry.getKey())) {
                    System.out.println(entry.getKey());
                    String imageStr = null;
                    String imageKey = null;
                    if (entry.getValue() instanceof JSONObject) {
                        if (null != ((JSONObject) entry.getValue()).getString("bar")) {
                            imageStr = ((JSONObject) entry.getValue()).getString("bar");
                            imageKey = "bar";
                        } else {
                            imageStr = ((JSONObject) entry.getValue()).getString("v");
                            imageKey = "v";
                        }
                    }
                    if (null != imageStr && !"".equals(imageStr) && null != imageKey && !"".equals(imageKey)) {
                        // 将base64转为图片存在服务器，并将返回值置空
                        String path = FileUtils.GenerateImage(imageStr, entry.getKey());
                        TImage tImage = new TImage();
                        tImage.setName(entry.getKey());
                        tImage.setPath(path);
                        tImage.setCreateTime(new Date());
                        tImage.setThirdId(recordId);
                        imageService.insertTImage(tImage);
                        ((JSONObject) entry.getValue()).put(imageKey, "");
                    }
                }
                if (!(entry.getValue() instanceof String)) {
                    doRemoveBase64Image(entry.getValue(), entry.getKey(), imageKeys, recordId);
                }
            }
        }
    }

    /**
     * 添加记录
     *
     * @param patientId 患者id
     * @return {@code AjaxResult}
     */
    @ApiOperation("新增预约记录")
    @Log(title = "App新增预约记录", businessType = BusinessType.INSERT)
    @PostMapping("/record/add/{patientId}")
    public AjaxResult addRecord(@PathVariable("patientId") Long patientId)
    {
        TRecord tRecord = new TRecord();
        tRecord.setHospitalId(SecurityUtils.getLoginUser().getUser().getHospitalId());
        tRecord.setPatientId(patientId);
        tRecord.setDoctorId(getUserId());
        tRecord.setStatus("0");
        // 新增预约限制：同一个医生 一天只能预约一次；
        tRecord.setAppointmentTime(DateUtils.dateTime(new Date()));
        List<TRecord> tRecords = tRecordService.selectTRecordList(tRecord);
        if (!CollectionUtils.isEmpty(tRecords)) {
            String appointmentTime = DateUtils.parseDateToStr(DateUtils.YYYY_MM_DD_HH_MM_SS ,tRecords.get(0).getCreateTime());
            return AjaxResult.error("同一个医生，一天只能预约一次；您今天已经在" + appointmentTime + "预约");
        }
        tRecord.setCreateBy(getUsername());
        tRecordService.insertTRecord(tRecord);
        TPatientHospitalDoctor tPatientHospitalDoctor = new TPatientHospitalDoctor();
        tPatientHospitalDoctor.setHospitalId(tRecord.getHospitalId());
        tPatientHospitalDoctor.setDoctorId(tRecord.getDoctorId());
        tPatientHospitalDoctor.setPatientId(tRecord.getPatientId());
        List<TPatientHospitalDoctor> tPHDs = itPatientHospitalDoctorService.selectTPatientHospitalDoctorList(tPatientHospitalDoctor);
        if (tPHDs.isEmpty())
        {
            itPatientHospitalDoctorService.insertTPatientHospitalDoctor(tPatientHospitalDoctor);
        }
        return AjaxResult.success("预约成功。");
    }

    /**
     * 查询医生诊断数量
     *
     * @param doctorId 医生id
     * @return {@code AjaxResult}
     */
    @ApiOperation("查询医生诊断数量")
    @Log(title = "APP查询医生诊断数量", businessType = BusinessType.OTHER)
    @GetMapping("/doctor/recordCount/{id}")
    public AjaxResult numberOfDiagnosis(@PathVariable("id") Long doctorId){
        return AjaxResult.success(tRecordService.queryDoctorNumberOfDiagnosis(doctorId));
    }

    /**
     * 查询患者诊断记录列表
     *
     * @param appPatientRecordReq 应用病人记录要求
     * @return {@code AjaxResult}
     */
    @ApiOperation("APP查询患者诊断记录列表")
    @Log(title = "APP查询患者诊断记录列表", businessType = BusinessType.OTHER)
    @GetMapping("/patient/record/list")
    public AjaxResult queryInPatientsWithDiagnosisOfRecordList(AppPatientRecordReq appPatientRecordReq){
        TRecordResultReq tRecordResultReq = new TRecordResultReq();
        BeanUtils.copyProperties(appPatientRecordReq, tRecordResultReq);
        return AjaxResult.success(tRecordService.selectTRecordListWithoutResult(tRecordResultReq));
    }

    /**
     * 查询版本信息
     *
     * @return {@code AjaxResult}
     */
    @ApiOperation("APP查询版本信息")
    @GetMapping("/version/info")
    public AjaxResult queryVersionInformation(){
        String localPath = RuoYiConfig.getAppInfoPath();
        String file = FileUtils.readJsonFile(localPath +"/" + "appVersion.json");
        return AjaxResult.success(JSONObject.parseObject(file));
    }
}
