package com.ruoyi.system.service.impl;

import com.alibaba.fastjson.JSONObject;
import com.ruoyi.common.core.domain.AjaxResult;
import com.ruoyi.common.utils.DateUtils;
import com.ruoyi.common.utils.StringUtils;
import com.ruoyi.system.domain.TPatientHospitalDoctor;
import com.ruoyi.system.domain.TRecord;
import com.ruoyi.system.domain.req.TRecordResultReq;
import com.ruoyi.system.domain.resp.PatientHistoryRecordResp;
import com.ruoyi.system.domain.resp.RecordHistoryDataResp;
import com.ruoyi.system.domain.resp.TRecordResp;
import com.ruoyi.system.mapper.TRecordMapper;
import com.ruoyi.system.service.ITRecordService;
import org.apache.commons.collections4.CollectionUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.*;
import java.util.stream.Collectors;

/**
 * 诊断记录信息Service业务层处理
 *
 * @author ruoyi
 * @date 2022-08-03
 */
@Service
public class TRecordServiceImpl implements ITRecordService
{
    private static final Logger logger = LoggerFactory.getLogger(TRecordServiceImpl.class);

    /**
     * AI返回结果中可能为空值的key
     */
    private static final List<String> nullValKeys = new ArrayList<String>(){{
        add("st_avg_yellow");
        add("sz_avg_purple");
        add("sz_avg_red");
        add("sx_vein_avg_purple");
    }};;

    @Autowired
    private TRecordMapper tRecordMapper;

    /**
     * 查询诊断记录信息
     *
     * @param id 诊断记录信息主键
     * @return 诊断记录信息
     */
    @Override
    public TRecord selectTRecordById(Long id)
    {
        return tRecordMapper.selectTRecordById(id);
    }

    /**
     * 查询诊断记录信息列表
     *
     * @param tRecord 诊断记录信息
     * @return 诊断记录信息
     */
    @Override
    public List<TRecord> selectTRecordList(TRecord tRecord)
    {
        return tRecordMapper.selectTRecordList(tRecord);
    }

    /**
     * 新增诊断记录信息
     *
     * @param tRecord 诊断记录信息
     * @return 结果
     */
    @Override
    public int insertTRecord(TRecord tRecord)
    {
        if (null == tRecord.getCreateTime()) {
            tRecord.setCreateTime(DateUtils.getNowDate());
        }
        return tRecordMapper.insertTRecord(tRecord);
    }

    /**
     * 修改诊断记录信息
     *
     * @param tRecord 诊断记录信息
     * @return 结果
     */
    @Override
    public int updateTRecord(TRecord tRecord)
    {
        tRecord.setUpdateTime(DateUtils.getNowDate());
        return tRecordMapper.updateTRecord(tRecord);
    }

    /**
     * 批量删除诊断记录信息
     *
     * @param ids 需要删除的诊断记录信息主键
     * @return 结果
     */
    @Override
    public int deleteTRecordByIds(Long[] ids)
    {
        return tRecordMapper.deleteTRecordByIds(ids);
    }

    /**
     * 删除诊断记录信息信息
     *
     * @param id 诊断记录信息主键
     * @return 结果
     */
    @Override
    public int deleteTRecordById(Long id)
    {
        return tRecordMapper.deleteTRecordById(id);
    }

    @Override
    public List<TRecordResp> selectTRecordListWithoutResult(TRecordResultReq tRecord)
    {
        return tRecordMapper.selectTRecordListWithoutResult(tRecord);
    }

    @Override
    public List<TRecordResp> queryHomePatientList(TRecordResultReq tRecord) {
        List<TRecordResp> tRecordResps = tRecordMapper.selectHomePatientList(tRecord);
        return tRecordResps;
    }

    @Override
    public TRecordResp selectById(Long id)
    {
        return tRecordMapper.selectById(id);
    }

    @Override
    public List<TRecordResp> queryRecordListByPHDids(TPatientHospitalDoctor tPatientHospitalDoctor)
    {
        return tRecordMapper.queryRecordListByPHDids(tPatientHospitalDoctor);
    }

    @Override
    public AjaxResult getHistoryList(TRecord tRecord)
    {
        logger.info("get history list param is : {} ", tRecord);
        List<TRecord> tRecords = tRecordMapper.selectTRecordList(tRecord);
        tRecords.removeIf(o->"0".equals(o.getStatus()));
        if (CollectionUtils.isEmpty(tRecords)){
            return AjaxResult.success();
        }
        List<TRecord> records = tRecords.stream().sorted(Comparator.comparing(TRecord::getCreateTime)).collect(Collectors.toList());
        List<String> jsonKeyList = new ArrayList<String>(){{
            add("img_sz");
            add("img_sz_yudian");
            add("img_sz_yuban");
            add("img_sz_liewen");
            add("img_st");
            add("img_sx");
            add("img_sx_vein");
        }};
        RecordHistoryDataResp recordHistoryDataRespMap = getDataList(records,jsonKeyList);
        return AjaxResult.success(recordHistoryDataRespMap);
    }

    private RecordHistoryDataResp getDataList(List<TRecord> records, List<String> jsonKeyList)
    {
        RecordHistoryDataResp recordHistoryDataResp = new RecordHistoryDataResp();
        // 构造时间轴坐标值
        List<Date> dateList = records.stream().map(TRecord -> {
            if (null != TRecord.getUploadTime()) {
                return TRecord.getUploadTime();
            } else {
                return TRecord.getCreateTime();
            }
        }).collect(Collectors.toList());
        recordHistoryDataResp.setDateList(dateList);
        // 构造纵坐标值
        List<JSONObject> result = records.stream().map(tRecord -> {
            JSONObject jsonObject = new JSONObject();
            if(StringUtils.isEmpty(tRecord.getUpdateResult())){
                jsonObject = JSONObject.parseObject(tRecord.getAiResult());
            }else if(!StringUtils.isEmpty(tRecord.getUpdateResult())){
                jsonObject = JSONObject.parseObject(tRecord.getUpdateResult());
            }
            return jsonObject;
        }).filter(Objects::nonNull).collect(Collectors.toList());
        if (!CollectionUtils.isEmpty(result)){
            HashMap<String, ArrayList<Object>> map = new HashMap<>();
            result.forEach(jsonObject -> {
                getJsonData(jsonObject.getJSONObject("data"), "data", map, jsonKeyList);
            });
            recordHistoryDataResp.setHistoryDataMap(map);
        }
        return recordHistoryDataResp;
    }

    /**
     * 获取json数据
     *
     * @param obj            obj
     * @param listName       列表名称
     * @param historyDataMap 历史数据
     * @param removeKeyList  删除键列表
     */
    private void getJsonData(Object obj, String listName, HashMap<String, ArrayList<Object>> historyDataMap, List<String> removeKeyList) {
        // aiResult结果集中只包括JSONObject,不包括JSONArray
        if (obj instanceof JSONObject) {
            // "sx_vein_avg_purple": {}
            // "sx_vein_avg_purple": {"anomaly": 0}
            // 这两种情况应该添加默认值
            if (nullValKeys.contains(listName) && null == ((JSONObject) obj).get("v")) {
                // 加默认值
                doSetHistoryData(historyDataMap, listName, "0");
            }
            for (Map.Entry<String, Object> entry : ((JSONObject) obj).entrySet()) {
                if ((entry.getValue() instanceof JSONObject)) {
                    getJsonData(entry.getValue(), entry.getKey(), historyDataMap, removeKeyList);
                } else {
                    // 忽略带图片字段
                    if (removeKeyList.contains(listName)) {
                        continue;
                    }
                    //sz_avg_purple: { "anomaly": 0, "v": "正常" }
                    // anomaly属于预留字段，暂时无用; 各个任务中的字段v是任务实际值
                    if (!"v".equals(entry.getKey())) {
                        continue;
                    }
                    doSetHistoryData(historyDataMap, listName, entry.getValue());
                }
            }
        }
    }

    /**
     * 做历史数据
     *
     * @param historyDataMap 历史数据map
     * @param listName       列表名称
     * @param val            值
     */
    private void doSetHistoryData(HashMap<String, ArrayList<Object>> historyDataMap, String listName, Object val) {
        ArrayList<Object> list;
        if (null == historyDataMap.get(listName)) {
            list = new ArrayList<>();
        } else {
            list = historyDataMap.get(listName);
        }
        list.add(val);
        historyDataMap.put(listName, list);
    }

    /**
     * 查询医生诊断次数
     *
     * @param doctorId 医生id
     * @return int
     */
    @Override
    public int queryDoctorNumberOfDiagnosis(Long doctorId) {
        return tRecordMapper.queryDoctorNumberOfDiagnosis(doctorId);
    }

    @Override
    public List<PatientHistoryRecordResp> queryPatientHistoryRecord(TRecord tRecord) {
        return tRecordMapper.queryPatientHistoryRecord(tRecord);
    }


}
